﻿using Ranorex;
using Ranorex.Core.Reporting;
using Ranorex.Core.Testing;
using System;

namespace ReportToPDF
{
	/// <summary>
	/// Allows conversion of Ranorex report files to PDF
	/// </summary>
	[TestModule("FFA0759D-37D2-4ABB-89A7-411F0FCF2DFE", ModuleType.UserCode, 1)]
	public class ReportToPDF : ITestModule
	{
		//Variables
		string PDFName;
		string xml;
		string details;
		static bool registered = false;
		
		/// <summary>
		/// Constructs a new instance.
		/// </summary>
		public ReportToPDF()
		{
			//Init variables
			this.PDFName = "";
			
			this.xml =  "";
			
			//Possible values: none | failed | all
			this.details = "all";
		}
		
		/// <summary>
		/// Performs the playback of actions in this module.
		/// </summary>
		/// <remarks>You should not call this method directly, instead pass the module
		/// instance to the <see cref="TestModuleRunner.Run(ITestModule)"/> method
		/// that will in turn invoke this method.</remarks>
		void ITestModule.Run()
		{
			//Delegate must be registered only once
			if(!registered)
			{
				//PDF will be generated at the very end of the testsuite
				TestSuite.TestSuiteCompleted += delegate {
					
					//Specify report name if not already set
					if(String.IsNullOrEmpty(this.PDFName))
					{
						this.PDFName = CreatePDFName();
					}
					
					//Necessary to end the testreport in order to update the duration
					TestReport.EndTestModule();
					
					//Comment out if ConvertReportToPDF() is called directly
					try
					{
						Report.LogHtml(ReportLevel.Success,"PDFReport", "Successfully created PDF: <a href='" + ConvertReportToPDF(PDFName, xml, details) + "' target='_blank'>Open PDF</a>");
					}
					catch (Exception e)
					{
						Console.BackgroundColor = ConsoleColor.Black;
						Console.ForegroundColor = ConsoleColor.Red;
						Console.WriteLine("ReportToPDF: " + e.Message);
						Console.ResetColor();
						Console.WriteLine("Press any key to continue...");
						Console.ReadKey();
					}
					
					//Delete *.rxzlog if not enabled within test suite settings
					Cleanup();
					
					//Update error value
					UpdateError();
				};
				
				registered = true;
			}
		}
		
		public static string ConvertReportToPDF (string PDFName, string xml, string details)
		{
			var reportFileDirectory = TestReport.ReportEnvironment.ReportFileDirectory;
			var name = TestReport.ReportEnvironment.ReportName;
			
			var input = TestReport.ReportEnvironment.ReportZipFilePath;
			var PDFReportFilePath = String.Format(@"{0}\{1}", reportFileDirectory, CheckExtension(PDFName));
			
			TestReport.SaveReport();
			Report.Zip(TestReport.ReportEnvironment, reportFileDirectory, name);

			Ranorex.PDF.Creator.CreatePDF(input, PDFReportFilePath, xml, details);
			return PDFReportFilePath;
		}

		private static string CheckExtension(string PDFName)
		{
			var split = PDFName.Split('.');
			
			for(int i =0; i <split.Length; i++)
			{
				if(split[i].Contains("pdf") && i == split.Length -1)
				{
					return PDFName;
				}
			}
			
			return PDFName + ".pdf";
		}
		
		private void Cleanup()
		{
			TestSuite a = (TestSuite) TestSuite.Current;
			var isCompressedReport = a.ReportCompress;

			if(!isCompressedReport)
			{
				try
				{
					if(System.IO.File.Exists(TestReport.ReportEnvironment.ReportZipFilePath))
					{
						System.IO.File.Delete(TestReport.ReportEnvironment.ReportZipFilePath);
					}
				}
				catch (Exception e)
				{
					Report.Warn("Failed to delete *.rxzlog: " + e.Message);
				}
			}
		}
		
		private string CreatePDFName()
		{
			//Report Status is not part of the ReportName at this stage of the test
			var name = TestReport.ReportEnvironment.ReportName;

			//Get status from TestSuite
			var testsuite = (TestSuite) TestSuite.Current;
			
			if( testsuite.ReportFormatString.Contains("%X"))
			{
				name = name += "_" + GetTestSuiteStatus();
			}

			return name;
		}
		
		private void UpdateError()
		{
			var testsuite = (TestSuite) TestSuite.Current;
			
			if(GetTestSuiteStatus().Contains("Failed"))
			{
				Report.Failure("Rethrow Exception within PDF Module (Necessary for correct error value)");
			}
		}
		
		private string GetTestSuiteStatus()
		{
			string status = "";
			
			var rootChildren = ActivityStack.Instance.RootActivity.Children;
			
			if(rootChildren.Count > 1)
			{
				Console.WriteLine("Multiple TestSuiteActivites, status taken from first entry");
			}
			
			var testSuiteAct = rootChildren[0] as TestSuiteActivity;
			
			if(testSuiteAct != null)
			{
				status = testSuiteAct.Status.ToString();
			}
			
			return status;
		}
	}
}
